@@ -33,7 +33,7 @@ def get_cygwin_dir():
 
     raise ValueError("Cygwin directory unknown; please call lifelib.add_cygdir(r'D:\\path\\to\\cygwin64') or lifelib.install_cygwin()")
 
-def install_cygwin(root_directory=None, packages=['python2', 'python2-numpy', 'python3', 'python3-numpy', 'gcc-core', 'gcc-g++', 'make', 'git']):
+def install_cygwin(root_directory=None, packages=['python38-numpy', 'gcc-core', 'gcc-g++', 'make', 'git']):
     '''
     For lifelib to run correctly on Windows, there needs to be a 64-bit Cygwin
     installation with g++, Python, and numpy. The user is free to download and
@@ -112,7 +112,7 @@ def verify_installation():
 
     print("Verifying numpy is installed...")
     if (os.name == 'nt'):
-        np_version = subprocess.check_output(get_local_bash() + ['python', '-c', 'import numpy as np; print(np.__version__)'])
+        np_version = subprocess.check_output(get_local_bash() + ['python3', '-c', 'import numpy as np; print(np.__version__)'])
     else:
         import numpy as np
         np_version = np.__version__
@@ -132,7 +132,7 @@ def get_local_bash():
 def get_local_python():
 
     if (os.name == 'nt'):
-        return 'python'
+        return 'python3'
     else:
         return this_python
 
@@ -465,6 +465,7 @@ def generate_code(rules, clean_before=False):
 
         f.write('#pragma once\n')
         f.write('#include <string>\n')
+        f.write('#include <vector>\n')
 
         #include the generated code for each rule:
         for rulestring in rules:
@@ -494,6 +495,14 @@ def generate_code(rules, clean_before=False):
         f.write('        return -1;\n')
         f.write('    }\n\n')
 
+        # Function to return all rulestrings:
+        f.write('    std::vector<std::string> get_all_rules() {\n')
+        f.write('        std::vector<std::string> x;\n')
+        for r in rules:
+            f.write('        x.emplace_back("%s");\n' % r)
+        f.write('        return x;\n')
+        f.write('    }\n\n')
+
         # Function to obtain family code from rule integer:
         f.write('    int uli_get_family(int rule) {\n')
         f.write('        switch (rule) {\n')
