@@ -381,9 +381,114 @@ namespace apg {
         }
 
 
-        std::vector<bitworld> getclusters(bitworld &live, bitworld &env, bool rigorous) {
+        std::vector<std::string> pseudoBangBang(pattern pat, std::vector<bitworld> *clvec) {
+            /*
+            * Borrowed from apgmera, and upgraded.
+            */
 
-            (void) rigorous;
+            uint64_t period = pat.ascertain_period();
+            bool isOscillator = ((pat.dx == 0) && (pat.dy == 0));
+            pattern hist(&lh, "", rule + "History");
+            hist += pat;
+            hist = hist[period + 2];
+            bitworld lrem = hist.flatlayer(0);
+            bitworld env = hist.flatlayer(1);
+
+            // If we have a moving object, do not reiterate:
+            bool reiterate = isOscillator && (zoi.length() <= 2);
+
+            std::map<std::pair<int64_t, int64_t>, uint64_t> geography;
+            uint64_t label = 0;
+            while (lrem.population() != 0) {
+                bitworld cluster = grow_cluster(lrem.get1cell(), env, reiterate ? "9" : zoi);
+                lrem -= cluster;
+                label += 1;
+                std::vector<std::pair<int64_t, int64_t> > celllist = cluster.getcells();
+                for (uint64_t i = 0; i < celllist.size(); i++) {
+                    geography[celllist[i]] = label;
+                }
+            }
+
+            while (reiterate) {
+                reiterate = false;
+                for (uint64_t i = 0; i < period; i++) {
+                    hist = hist[1];
+                    bitworld lcurr = hist.flatlayer(0);
+                    bitworld dcurr = bleed(lcurr, "9");
+                    dcurr -= env;
+                    std::vector<std::pair<int64_t, int64_t> > liberties = dcurr.getcells();
+                    for (uint64_t j = 0; j < liberties.size(); j++) {
+                        int64_t ix = liberties[j].first;
+                        int64_t iy = liberties[j].second;
+                        std::map<uint64_t, uint64_t> tally;
+                        for (int64_t ux = 0; ux <= 2; ux++) {
+                            for (int64_t uy = 0; uy <= 2; uy++) {
+                                int value = geography[std::pair<int64_t, int64_t>(ux + ix - 1, uy + iy - 1)];
+                                if (lcurr.getcell(ux + ix - 1, uy + iy - 1)) {
+                                    tally[value] = tally[value] + (1 << (uy * 3 + ux));
+                                }
+                            }
+                        }
+
+                        uint64_t dominantColour = 0;
+                        std::map<uint64_t, uint64_t>::iterator it2;
+                        for (it2 = tally.begin(); it2 != tally.end(); it2++) {
+                            int colour = it2->first;
+                            uint64_t count = it2->second;
+                            if (transtable[count]) { dominantColour = colour; }
+                            // if (__builtin_popcountll(count) == 3) { dominantColour = colour; }
+                        }
+                        // Resolve dependencies:
+                        if (dominantColour != 0) {
+                            std::map<std::pair<int64_t, int64_t>, uint64_t>::iterator it3;
+                            for (it3 = geography.begin(); it3 != geography.end(); it3++) {
+                                std::pair<int64_t, int64_t> coords = it3->first;
+                                uint64_t colour = it3->second;
+
+                                if (tally[colour] > 0) {
+                                    geography[coords] = dominantColour;
+                                    if (colour != dominantColour) {
+                                        // A change has occurred; keep iterating until we achieve stability:
+                                        reiterate = true;
+                                    }
+                                }
+                            }
+                        }
+                    }
+                }
+            }
+
+            bitworld lcurr = (isOscillator ? pat.flatlayer(0) : hist.flatlayer(0));
+            std::vector<bitworld> cbs(label+1);
+            std::map<std::pair<int64_t, int64_t>, uint64_t>::iterator it3;
+            for (it3 = geography.begin(); it3 != geography.end(); it3++) {
+                std::pair<int64_t, int64_t> coords = it3->first;
+                uint64_t colour = it3->second;
+                cbs[colour].setcell(coords.first, coords.second, 1);
+            }
+            std::vector<std::string> components;
+            for (uint64_t l = 1; l <= label; l++) {
+                cbs[l] &= lcurr;
+                if (cbs[l].population() > 0) {
+                    if (clvec != 0) { clvec->push_back(cbs[l]); }
+                    apg::pattern ppart(lab, lab->demorton(cbs[l], 1), rule);
+                    components.push_back(ppart.apgcode());
+                }
+            }
+
+            for (auto it = components.begin(); it != components.end(); ++it) {
+                if ((*it) == "PATHOLOGICAL") {
+                    // invalid separation
+                    std::vector<std::string> newcomps;
+                    newcomps.push_back(pat.apgcode());
+                    return newcomps;
+                }
+            }
+
+            return components;
+        }
+
+        std::vector<bitworld> getclusters(bitworld &live, bitworld &env, bool rigorous) {
 
             bitworld lrem = live;
             std::vector<bitworld> clusters;
@@ -393,7 +498,12 @@ namespace apg {
                 bitworld cluster = grow_cluster(lrem.get1cell(), env, zoi);
                 cluster &= lrem;
                 lrem -= cluster;
-                clusters.push_back(cluster);
+                if (rigorous) {
+                    pattern ppart(lab, lab->demorton(cluster, 1), rule);
+                    pseudoBangBang(ppart, &clusters);
+                } else {
+                    clusters.push_back(cluster);
+                }
             }
 
             return clusters;
