@@ -6,6 +6,26 @@
 
 namespace apg {
 
+    int reportCudaError(cudaError_t err) {
+
+        if (err != cudaSuccess) {
+            std::cerr << "\033[31;1mCUDA Error " << ((int) err) << " : " <<
+                cudaGetErrorString(err) << "\033[0m" << std::endl;
+
+            return ((int) err);
+        }
+
+        return 0;
+    }
+
+    int getDeviceCount() {
+
+        int count = 0;
+        if (reportCudaError(cudaGetDeviceCount(&count))) { return -1; }
+        return count;
+
+    }
+
     void GpuSearcher::pump(std::string seed, uint64_t epoch, std::vector<uint64_t> &vec) {
 
         hash_container *hc = (hash_container*) this->xhc;
