@@ -22,6 +22,84 @@ _DI_ uint4 load_hash(unsigned int *hash_ptr) {
     return result;
 }
 
+_DI_ void hreflect_even(uint4 &b) {
+
+    b.x = (b.x >> 8) | (b.y << 24);
+    b.z = (b.z >> 8) | (b.w << 24);
+    b.y = __brev(b.x);
+    b.w = __brev(b.z);
+
+}
+
+_DI_ void hreflect_odd(uint4 &b) {
+
+    hreflect_even(b);
+    b.x <<= 1;
+    b.z <<= 1;
+
+}
+
+_DI_ void vreflect_even(uint4 &b) {
+
+    int laneId = threadIdx.x & 31;
+    int src = (laneId < 16) ? (27 - laneId) : (laneId - 4);
+
+    b.x = shuffle_32(b.x, src);
+    b.y = shuffle_32(b.y, src);
+    b.z = shuffle_32(b.z, src);
+    b.w = shuffle_32(b.w, src);
+
+    if (laneId < 16) {
+        uint32_t c = b.x; b.x = b.z; b.z = c;
+        c = b.y; b.y = b.w; b.w = c;
+    }
+}
+
+_DI_ void vreflect_odd(uint4 &b) {
+
+    int laneId = threadIdx.x & 31;
+    int src1 = (laneId < 16) ? (28 - laneId) : (laneId - 4);
+    int src2 = (laneId < 16) ? (27 - laneId) : (laneId - 4);
+
+    b.x = shuffle_32(b.x, src1);
+    b.y = shuffle_32(b.y, src1);
+    b.z = shuffle_32(b.z, src2);
+    b.w = shuffle_32(b.w, src2);
+
+}
+
+_DI_ void restrict_C2_4(uint4 &b) {
+
+    int laneId = threadIdx.x & 31;
+    if (laneId < 16) {
+        b.x = 0; b.z = 0;
+    } else {
+        b.y = 0; b.w = 0;
+    }
+}
+
+_DI_ void restrict_C2_2(uint4 &b) {
+
+    int laneId = threadIdx.x & 31;
+    if (laneId < 16) {
+        b.x = 0; b.z = 0;
+    } else {
+        b.y &= 1;
+        b.w &= 1;
+    }
+}
+
+_DI_ void restrict_C2_1(uint4 &b) {
+
+    int laneId = threadIdx.x & 31;
+    if (laneId < 16) {
+        b.x = 0; b.z = 0;
+    } else {
+        if (laneId > 16) { b.y &= 1; }
+        b.w &= 1;
+    }
+}
+
 /**
  * Advance a 64x64 torus by one generation using a single warp.
  *
@@ -59,18 +137,17 @@ _DI_ uint4 advance_torus(uint4 old) {
     }
 
     // 24 boolean ops + 4 shuffles:
-    uint4 sv; uint4 pt8;
+    uint4 xor3; uint4 xor3prime; uint4 sv; uint4 pt8;
+    xor3.x = xor2.x ^ old.x;
+    xor3.y = xor2.y ^ old.y;
+    xor3.z = xor2.z ^ old.z;
+    xor3.w = xor2.w ^ old.w;
+    xor3prime.x = shuffle_32(xor3.x, lowerthread);
+    xor3prime.y = shuffle_32(xor3.y, lowerthread);
+    xor3prime.z = shuffle_32(xor3.z, upperthread);
+    xor3prime.w = shuffle_32(xor3.w, upperthread);
     {
-        uint4 xor3; uint4 xor3prime; uint4 uda; uint4 udx;
-        xor3.x = xor2.x ^ old.x;
-        xor3.y = xor2.y ^ old.y;
-        xor3.z = xor2.z ^ old.z;
-        xor3.w = xor2.w ^ old.w;
-        xor3prime.x = shuffle_32(xor3.x, lowerthread);
-        xor3prime.y = shuffle_32(xor3.y, lowerthread);
-        xor3prime.z = shuffle_32(xor3.z, upperthread);
-        xor3prime.w = shuffle_32(xor3.w, upperthread);
-
+        uint4 uda; uint4 udx;
         udx.x = xor3.z ^ xor3prime.z;
         udx.y = xor3.w ^ xor3prime.w;
         udx.z = xor3.x ^ xor3prime.x;
@@ -91,27 +168,24 @@ _DI_ uint4 advance_torus(uint4 old) {
     }
 
     // 32 boolean ops + 4 shuffles
-    uint4 pt4; uint4 xc3;
-    {
-        uint4 maj3; uint4 maj3prime;
-        maj3.x = sh.x | (old.x & xor2.x);
-        maj3.y = sh.y | (old.y & xor2.y);
-        maj3.z = sh.z | (old.z & xor2.z);
-        maj3.w = sh.w | (old.w & xor2.w);
-        maj3prime.x = shuffle_32(maj3.x, lowerthread);
-        maj3prime.y = shuffle_32(maj3.y, lowerthread);
-        maj3prime.z = shuffle_32(maj3.z, upperthread);
-        maj3prime.w = shuffle_32(maj3.w, upperthread);
+    uint4 maj3; uint4 maj3prime; uint4 pt4; uint4 xc3;
+    maj3.x = sh.x | (old.x & xor2.x);
+    maj3.y = sh.y | (old.y & xor2.y);
+    maj3.z = sh.z | (old.z & xor2.z);
+    maj3.w = sh.w | (old.w & xor2.w);
+    maj3prime.x = shuffle_32(maj3.x, lowerthread);
+    maj3prime.y = shuffle_32(maj3.y, lowerthread);
+    maj3prime.z = shuffle_32(maj3.z, upperthread);
+    maj3prime.w = shuffle_32(maj3.w, upperthread);
 
-        pt4.x = (maj3prime.z ^ maj3.z) ^ (sh.x ^ sv.x);
-        pt4.y = (maj3prime.w ^ maj3.w) ^ (sh.y ^ sv.y);
-        pt4.z = (maj3prime.x ^ maj3.x) ^ (sh.z ^ sv.z);
-        pt4.w = (maj3prime.y ^ maj3.y) ^ (sh.w ^ sv.w);
-        xc3.x = (maj3prime.z | maj3.z) ^ (sh.x | sv.x);
-        xc3.y = (maj3prime.w | maj3.w) ^ (sh.y | sv.y);
-        xc3.z = (maj3prime.x | maj3.x) ^ (sh.z | sv.z);
-        xc3.w = (maj3prime.y | maj3.y) ^ (sh.w | sv.w);
-    }
+    pt4.x = (maj3prime.z ^ maj3.z) ^ (sh.x ^ sv.x);
+    pt4.y = (maj3prime.w ^ maj3.w) ^ (sh.y ^ sv.y);
+    pt4.z = (maj3prime.x ^ maj3.x) ^ (sh.z ^ sv.z);
+    pt4.w = (maj3prime.y ^ maj3.y) ^ (sh.w ^ sv.w);
+    xc3.x = (maj3prime.z | maj3.z) ^ (sh.x | sv.x);
+    xc3.y = (maj3prime.w | maj3.w) ^ (sh.y | sv.y);
+    xc3.z = (maj3prime.x | maj3.x) ^ (sh.z | sv.z);
+    xc3.w = (maj3prime.y | maj3.y) ^ (sh.w | sv.w);
 
     // 12 boolean ops
     uint4 newstate;
@@ -120,6 +194,14 @@ _DI_ uint4 advance_torus(uint4 old) {
     newstate.z = pt4.z & xc3.z & (pt8.z | old.z);
     newstate.w = pt4.w & xc3.w & (pt8.w | old.w);
 
+    #ifdef PEDESTRIAN_LIFE
+    // correction for eighth birth transition
+    newstate.x |= (maj3prime.z & maj3.z & xor3.z & xor3prime.z & sh.x &~ old.x);
+    newstate.y |= (maj3prime.w & maj3.w & xor3.w & xor3prime.w & sh.y &~ old.y);
+    newstate.z |= (maj3prime.x & maj3.x & xor3.x & xor3prime.x & sh.z &~ old.z);
+    newstate.w |= (maj3prime.y & maj3.y & xor3.y & xor3prime.y & sh.w &~ old.w);
+    #endif
+
     return newstate;
 
 }
@@ -249,13 +331,18 @@ _DI_ int advance_initial(uint4 &a) {
     // the pattern to escape the 52x52 bounding box in this time period,
     // and only 1.6% of soups stabilise this quickly so we do not give
     // up much by skipping the stabilisation checks:
-    for (int i = 0; i < 10; i++) {
-        b = advance_torus(b);
-        b = advance_torus(b);
+    #ifdef C1_SYMMETRY
+    #define initial_gens 30
+    #else
+    #define initial_gens 10
+    #endif
+
+    #pragma unroll 2
+    for (int i = 0; i < initial_gens; i++) {
         b = advance_torus(b);
     }
 
-    int age = 24;
+    int age = initial_gens - 6;
 
     // Run until generation 600, checking for bounding box escapement
     // and 2-periodicity. This loop should fit comfortably in the GPU
@@ -464,6 +551,29 @@ __global__ void exhaustFirstTile(uint32_cu *hashes, uint32_cu *interesting, uint
     uint32_t *thishash = hashes + (pos << 3);
 
     uint4 a = load_hash(thishash);
+
+    #ifdef HREFLECT_EVEN
+    hreflect_even(a);
+    #endif
+    #ifdef HREFLECT_ODD
+    hreflect_odd(a);
+    #endif
+    #ifdef VREFLECT_EVEN
+    vreflect_even(a);
+    #endif
+    #ifdef VREFLECT_ODD
+    vreflect_odd(a);
+    #endif
+    #ifdef RESTRICT_C2_1
+    restrict_C2_1(a);
+    #endif
+    #ifdef RESTRICT_C2_2
+    restrict_C2_2(a);
+    #endif
+    #ifdef RESTRICT_C2_4
+    restrict_C2_4(a);
+    #endif
+
     int gencount = advance_initial(a);
 
     interesting[pos] = gencount;
