@@ -13,48 +13,22 @@
 #include <iostream>
 #include <unordered_map>
 
-#include "strashtable.hpp"
+#include "../../../cpads/include/cpads/indextable.hpp"
 
 
 namespace apg {
 
-    const static uint32_t klowbits = 8;
+template <typename K, typename I, typename V>
+struct kiventry {
 
-    template <typename T, typename I, int B>
-    class metaarray {
+    K key;
+    I next;
+    I gcflags; // For gc and ensuring nice standard layout.
+    V value;
 
-        std::vector<T*> arraylist;
+};
 
-        public:
-
-        I totalnodes = 0;
-
-        T* ind2ptr(I index) {
-            return (arraylist[index >> B] + (index & ((1 << B) - 1)));
-        }
-
-        T* newnode() {
-            if ((totalnodes & ((1 << B) - 1)) == 0) {
-                T* nextarray = (T*) zalloc(sizeof(T) << B);
-                arraylist.push_back(nextarray);
-            }
-            return ind2ptr(totalnodes++);
-        }
-
-        explicit metaarray() = default;
-
-        ~metaarray() {
-
-            while (!arraylist.empty()) {
-                free(arraylist.back());
-                arraylist.pop_back();
-            }
-
-        }
-
-    };
-
-    template <typename K, typename V, typename I=uint32_t>
+    template <typename K, typename V>
     class indirected_map {
         /*
         * Similar to an unordered_map, but the values are contiguous in
@@ -63,54 +37,57 @@ namespace apg {
 
         public:
 
-        metaarray<V, I, 5> elements;
+        hh::ivector<V, 5> elements;
         std::unordered_map<K, V*> hashtable;
 
         V& operator[](K key) {
 
             V** pointer_to_pointer = &(hashtable[key]);
             if (*pointer_to_pointer == 0) {
-                *pointer_to_pointer = elements.newnode();
+                *pointer_to_pointer = &(elements[elements.newnode()]);
             }
             return **pointer_to_pointer;
 
         }
 
+        uint64_t size() const { return elements.size(); }
 
     };
 
     template <typename K, typename I, typename V>
-    class kivtable : public strashtable<K, I, V> {
+    struct kivtable : public hh::indextable<kiventry<K, I, V>, K, kivtable<K, I, V>> {
 
-        public:
-
-        uint64_t total_bytes() const {
-            uint64_t nodemem = sizeof(kiventry<K, I, V>) * this->size();
-            uint64_t hashmem = sizeof(I) * this->hashsize();
-            return (nodemem + hashmem);
+        K compute_key(const kiventry<K, I, V> &entry) const {
+            return entry.key;
         }
 
         // Get node index from key:
         I getnode(const K &key, bool makenew) {
 
-            V blank;
-            memset(&blank, 0, sizeof(blank));
-
             if (makenew) {
-                return this->template touchnode<true, false>(key, blank);
+                kiventry<K, I, V> blank;
+                memset(&blank, 0, sizeof(blank));
+                blank.key = key;
+                return this->insert(blank, false);
             } else {
-                return this->template touchnode<false, false>(key, blank);
+                return this->find(key);
             }
         }
 
+        kiventry<K, I, V>* ind2ptr(I index) { return &(this->contents[index]); }
+
         // Create a (key, value) pair and return index:
         I setnode(const K &key, const V &value) {
 
-            return this->template touchnode<true, true>(key, value);
+            kiventry<K, I, V> blank;
+            memset(&blank, 0, sizeof(blank));
+            blank.key = key;
+            blank.value = value;
 
+            return this->insert(blank, true);
         }
 
-        explicit kivtable() : strashtable<K, I, V>(4096) { }
+        explicit kivtable() : hh::indextable<kiventry<K, I, V>, K, kivtable<K, I, V>>(1, 4096) { }
 
     };
 
