@@ -794,6 +794,15 @@ observer.observe(targetNode, config);
 
         return bbox
 
+    @property
+    def firstcell(self):
+        if self.empty():
+            return [0, 0]
+        y = self._getbound(1)
+        toprow = self[:,:y+1]
+        x = toprow._getbound(0)
+        return [x, y]
+
     @property
     def bounding_box(self):
 
@@ -928,7 +937,7 @@ observer.observe(targetNode, config);
 
         return float(centre[0]), float(centre[1]), float(pixelsize)
 
-    def make_gif_internal(self, p, disp, hue='#ff0000', width=350, height=350, border=5, bg=None):
+    def make_gif_internal(self, p, disp, hue='#000000', width=350, height=350, border=5, bg=None):
 
         import numpy as np
         saturation = 1.0
@@ -958,7 +967,7 @@ observer.observe(targetNode, config);
         return anim2
         
 
-    def make_gif(self, filename=None, fps=25, **kwargs):
+    def make_gif(self, filename=None, fps=25, reorient=True, **kwargs):
         '''
         Intelligently create an animated GIF of an oscillator or spaceship.
         Spaceships will be oriented to point between north and north-west.
@@ -970,17 +979,19 @@ observer.observe(targetNode, config);
         disp = list(d['displacement'])
         p = d['period']
 
-        if abs(disp[0]) > abs(disp[1]):
-            y = y.transform('swap_xy')
-            disp = disp[::-1]
+        if reorient:
 
-        if disp[0] > 0:
-            y = y.transform('flip_x')
-            disp[0] = -disp[0]
+            if abs(disp[0]) > abs(disp[1]):
+                y = y.transform('swap_xy')
+                disp = disp[::-1]
 
-        if disp[1] > 0:
-            y = y.transform('flip_y')
-            disp[1] = -disp[1]
+            if disp[0] > 0:
+                y = y.transform('flip_x')
+                disp[0] = -disp[0]
+
+            if disp[1] > 0:
+                y = y.transform('flip_y')
+                disp[1] = -disp[1]
 
         bg = None
 
@@ -991,17 +1002,28 @@ observer.observe(targetNode, config);
             periods = 1 + (4 * (rect[2] + rect[3])) // p
             y = y[p * periods]
 
-            if (rect[3] > rect[2]):
+            if reorient and (rect[3] > rect[2]):
                 y = y.transform('swap_xy')
                 bg = bg.transform('swap_xy')
 
-        gif = y.make_gif_internal(p, disp, bg=bg, **kwargs)
+        gifdata = y.make_gif_internal(p, disp, bg=bg, **kwargs)
 
         if filename is None:
-            return gif
+            return gifdata
 
         import imageio
-        imageio.mimsave(filename, gif, fps=fps)
+        imageio.mimsave(filename, gifdata, fps=fps)
 
         # return the pattern that was giffed:
         return y
+
+
+    def gif(self, filename=None, **kwargs):
+
+        if filename is None:
+            filename = self.session.newfloat('anim') + '.gif'
+
+        self.make_gif(filename, **kwargs)
+
+        from IPython.display import Image
+        return Image(filename)
