@@ -101,7 +101,6 @@ class Lifetree(object):
         '''
 
         filename = os.path.abspath(filename)
-        tempfile = os.path.abspath(tempfile)
 
         if not os.path.isfile(filename):
             raise OSError("%s does not exist or is not a regular file" % filename)
@@ -114,6 +113,8 @@ class Lifetree(object):
             from importlib import import_module
             decompressor = import_module(compressed)
 
+            tempfile = os.path.abspath(tempfile)
+
             import shutil
             with decompressor.open(filename, 'rb') as f_in:
                 with open(tempfile, 'wb') as f_out:
@@ -141,18 +142,11 @@ class Lifetree(object):
 
     def pattern(self, rle="", rule=None, tempfile='tempfile', verify_apgcode=True):
 
-        tempfile = os.path.abspath(tempfile)
         is_apgcode = False
 
         if ('=' in rle) or ('[' in rle):
-            # Headerful RLE/MC; save and reload:
-            with open(tempfile, 'w') as f:
-                f.write(rle)
-            pptr = self.lifelib('CreatePatternFromFile', self.ptr, tempfile)
-            try:
-                os.remove(tempfile)
-            except OSError:
-                pass
+            # Headerful RLE/MC:
+            pptr = self.lifelib('CreatePatternFromFileContents', self.ptr, rle)
         else:
             # Headerless RLE:
             if rule is None:
