@@ -52,63 +52,10 @@ namespace apg {
         transpose8rS32(A+17, 2, 2, B+17);
     }
 
-    // Produce a SHA-256 hash of a string, and use it to generate a soup:
-    bitworld hashsoup_inner(std::string prehash, std::string symmetry) {
-        bool verbose = false;
-        if (prehash.front() == '@') {
-            prehash.erase(0, 1);
-            verbose = true;
-        }
-
-        uint8_t digest[32];
-        memset(digest, 0, 32);
-
-        SHA256 ctx = SHA256();
-        ctx.init();
-        ctx.update( (unsigned char*)prehash.c_str(), prehash.length());
-        ctx.final(digest);
-
-        uint8_t tsegid[32];
-        memset(tsegid, 0, 32);
-
-        if ((symmetry == "C2_2") && (prehash.substr(0, 2) != "l_")) {
-            // Correction for pre-apgluxe behaviour:
-            std::memcpy(tsegid, digest, 32);
-            transpose16(tsegid, digest);
-        } else {
-            transpose16(digest, tsegid);
-        }
-
-        if ((symmetry == "D2_xo") || (symmetry == "D2_x") || (symmetry == "D8_4") || (symmetry == "D8_1") || (symmetry == "D4_x4") || (symmetry == "D4_x1")) {
-            // We make our arrays diagonally symmetric:
-            uint8_t diggid[32];
-            memset(diggid, 0, 32);
-            for (int i = 0; i < 8; i++) {
-                diggid[2*i] = (digest[2*i] & ((1 << (8 - i)) - 1)) | (tsegid[2*i] & (256 - (1 << (8 - i))));
-                diggid[2*i + 17] = (digest[2*i + 17] & ((1 << (8 - i)) - 1)) | (tsegid[2*i + 17] & (256 - (1 << (8 - i))));
-                diggid[2*i + 1] = digest[2*i + 1];
-                diggid[2*i + 16] = tsegid[2*i + 16];
-            }
-
-            for (int i = 0; i < 32; i++) {
-                tsegid[i] ^= (diggid[i] ^ digest[i]);
-            }
-            std::memcpy(digest, diggid, 32);
-        }
+    bitworld hashsoup_mateon(std::string prehash, std::string symmetry, uint8_t *digest, bool verbose) {
 
         bitworld bw;
 
-        if ((symmetry == "8x32") || (symmetry == "4x64") || (symmetry == "2x128") || (symmetry == "1x256")) {
-            int height = symmetry[0] - 48;
-            int eighthwidth = 32 / height;
-            for (int j = 0; j < height; j++) {
-                for (int i = 0; i < eighthwidth; i++) {
-                    bw.world[std::pair<int32_t, int32_t>(eighthwidth - 1 - i, 0)] |= (((uint64_t) digest[eighthwidth*j+i]) << (8*j));
-                }
-            }
-            return bw;
-        }
-
         if ((symmetry == "Mateon1_32x32_Test") || (symmetry == "Mateon1_64x64_Test") || (symmetry == "Mateon1_128x128_Test") ||
             (symmetry == "Mateon1_256x256_Test") || (symmetry == "Mateon1_512x512_Test") || (symmetry == "Mateon1_1k_Test") ||
             (symmetry == "Mateon1_2k_Test") || (symmetry == "Mateon1_4k_Test") || (symmetry == "Mateon1_8k_Test")) {
@@ -149,7 +96,6 @@ namespace apg {
                         bw.world[std::pair<int32_t, int32_t>(i, j)] |= (((uint64_t) digest[b + 8*(i%4)] << (8 * b)));
                 }
             }
-            return bw;
         }
 
         if (symmetry == "Mateon1_Glider8_4_5_Test" || symmetry == "Mateon1_Glider6_5_6_Test") {
@@ -264,6 +210,69 @@ repeat:
                     std::cout << std::endl;
                 }
             }
+        }
+
+        return bw;
+    }
+
+    // Produce a SHA-256 hash of a string, and use it to generate a soup:
+    bitworld hashsoup_inner(std::string prehash, std::string symmetry) {
+        bool verbose = false;
+        if (prehash.front() == '@') {
+            prehash.erase(0, 1);
+            verbose = true;
+        }
+
+        uint8_t digest[32];
+        memset(digest, 0, 32);
+
+        SHA256 ctx = SHA256();
+        ctx.init();
+        ctx.update( (unsigned char*)prehash.c_str(), prehash.length());
+        ctx.final(digest);
+
+        if (symmetry.substr(0, 8) == "Mateon1_") {
+            return hashsoup_mateon(prehash, symmetry, digest, verbose);
+        }
+
+        uint8_t tsegid[32];
+        memset(tsegid, 0, 32);
+
+        if ((symmetry == "C2_2") && (prehash.substr(0, 2) != "l_")) {
+            // Correction for pre-apgluxe behaviour:
+            std::memcpy(tsegid, digest, 32);
+            transpose16(tsegid, digest);
+        } else {
+            transpose16(digest, tsegid);
+        }
+
+        if ((symmetry == "D2_xo") || (symmetry == "D2_x") || (symmetry == "D8_4") || (symmetry == "D8_1") || (symmetry == "D4_x4") || (symmetry == "D4_x1")) {
+            // We make our arrays diagonally symmetric:
+            uint8_t diggid[32];
+            memset(diggid, 0, 32);
+            for (int i = 0; i < 8; i++) {
+                diggid[2*i] = (digest[2*i] & ((1 << (8 - i)) - 1)) | (tsegid[2*i] & (256 - (1 << (8 - i))));
+                diggid[2*i + 17] = (digest[2*i + 17] & ((1 << (8 - i)) - 1)) | (tsegid[2*i + 17] & (256 - (1 << (8 - i))));
+                diggid[2*i + 1] = digest[2*i + 1];
+                diggid[2*i + 16] = tsegid[2*i + 16];
+            }
+
+            for (int i = 0; i < 32; i++) {
+                tsegid[i] ^= (diggid[i] ^ digest[i]);
+            }
+            std::memcpy(digest, diggid, 32);
+        }
+
+        bitworld bw;
+
+        if ((symmetry == "8x32") || (symmetry == "4x64") || (symmetry == "2x128") || (symmetry == "1x256")) {
+            int height = symmetry[0] - 48;
+            int eighthwidth = 32 / height;
+            for (int j = 0; j < height; j++) {
+                for (int i = 0; i < eighthwidth; i++) {
+                    bw.world[std::pair<int32_t, int32_t>(eighthwidth - 1 - i, 0)] |= (((uint64_t) digest[eighthwidth*j+i]) << (8*j));
+                }
+            }
             return bw;
         }
 
@@ -280,7 +289,7 @@ repeat:
         }
 
         if ((symmetry == "C1") || (symmetry == "G1") || (symmetry == "D2_x")) {
-            #ifdef __AVX512F__
+            #ifdef LONG_TILES
             bw.world[std::pair<int32_t, int32_t>(1, 2)] = b;
             bw.world[std::pair<int32_t, int32_t>(2, 2)] = a;
             bw.world[std::pair<int32_t, int32_t>(1, 3)] = d;
