@@ -76,8 +76,9 @@ $3b3o3b2o3b2o3bo2bo7bo2bo12b3o$3bo2bo2b2o3b2o2bo3bo7bob2o12bo2bo$3bo
 
         pat = self.lt.pattern('xp40_33zxs48g69gvzx3021zy833')
         samples = pat.download_samples()
-        periods = [s[10000][-1000:1000, -1000:1000].period for s in samples['C2_2']]
-        self.assertEqual(periods[:20], [40]*20)
+        samples = list(samples['C2_2'])[:20]
+        periods = [s[10000][-1000:1000, -1000:1000].period for s in samples]
+        self.assertEqual(periods, [40]*20)
 
         synth = pat.download_synthesis()
         res = synth[1000]
