@@ -5,6 +5,15 @@ import os
 
 class TestRuleTrees(unittest.TestCase):
 
+    def test_wright(self):
+
+        sess = lifelib.load_rules('b3456s235678d1-7', 'b2s34678d', force_compile=True)
+        lt = sess.lifetree()
+        x = lt.pattern('6.F$3.A.GFG$.2ABA2H$A2BFA.G$ABDFH$2ADBHF$2.CAGD.H$2.2ADEH$3.2ADAHC$5.2BDB$5.3A2.H$10.H!', 'b3456s235678d1-7')
+        self.assertEqual(x.apgcode, 'xq24_0sc315ek8gzx1x2l1a_y0aagogzw11w2368e4_wgcco8zy223iu_xggz44x2')
+        y = lt.pattern('xq83_0h_4h0a', 'b2s34678d')
+        self.assertEqual(y.apgcode, 'xq83_0h_4h0a')
+
     def test_langtons_loops(self):
 
         filename = os.path.join(lifelib.lifelib_dir, 'rules', 'source', 'Langtons-Loops.table')
